/*
 * fda.c - Free Debug Allocator
 *
 *  For copying information please read LICENSE
 *  
 * This program is released under the GNU General Public
 * license. 
 * 
 * Copyright (C) 2008 Omega Development Team <dev@taazoo.net>
 * Copyright (C) 1997 Thomas Helvey <tomh@inxpress.net>
 *
 * 
 * Allow for a clear path to be visiable when debuging
 * memory allocations.
 *
 * 
 *           $Id: fda.h 1055 2005-04-11 21:55:39Z  $
 */


#ifndef __FDA_H__
#define __FDA_H__


extern size_t fda_get_byte_count(void);
extern size_t fda_get_block_count(void);
extern size_t fda_sizeof(const void* p);
extern void   fda_clear_refs(void);
extern void   fda_set_ref(const void* p);
extern void   fda_assert_refs(void);
extern int    fda_enum_locations(void (*enumfn)(const char*, int, int));
extern int    fda_enum_leaks(void (*enumfn)(const char*, int, size_t, void*));
extern void   fda_dump_hash(void (*enumfn)(int, int));
extern void   fda_set_byte_limit(size_t nbytes);
extern int    valid_ptr(const void* p, size_t size);

extern void* fda_malloc(size_t size, const char* file, int line);
extern void* fda_realloc(void* p, size_t size, const char* file, int line);
extern void* fda_calloc(size_t nelems, size_t size, const char* file, int line);
extern char* fda_strdup(const char* src, const char* file, int line);
extern void  fda_free(void* p);


#ifdef USE_FDA_DEFS

#define malloc(s)     fda_malloc((s), __FILE__, __LINE__)
#define realloc(p, s) fda_realloc((p), (s), __FILE__, __LINE__)
#define calloc(n, s)  fda_calloc((n), (s), __FILE__, __LINE__)
#define strdup(s)     fda_strdup((s), __FILE__, __LINE__)
#define free(p)       fda_free((p))

#endif


extern void fda_set_lowmem_handler(void (*fn)(void));
extern void fda_set_nomem_handler(void (*fn)(void));


#endif /* __FDA_H__ */
