/*
 *           OMEGA SECURITY SERVICES
 * 	     (c) 2008 Omega Dev Team
 *
 *
 *                 $Id: g_opm.c 1375 2008-09-09 07:28:12Z twitch $
 */


#include "stdinc.h"
#include "server.h"

#include "opm.h"
#include "opm_error.h"
#include "opm_types.h"
#include "compat.h"

#define NAME "g_opm"
#define AUTHOR "Omega Team"
#define VERSION "$Id: g_opm.c 1375 2008-09-09 07:28:12Z twitch $"

#define ARRAY_SIZEOF(x) (sizeof(x) / sizeof((x)[0]))

#if 0
	OPM_T *scanner;
#endif

pthread_t complete = 0;

void opm_entry(void *args);
void open_proxy(OPM_T *, OPM_REMOTE_T *, int, void *);
void negotiation_failed(OPM_T *, OPM_REMOTE_T *, int, void *);
void opm_timeout(OPM_T *, OPM_REMOTE_T *, int, void *);
void end(OPM_T *, OPM_REMOTE_T *, int, void *);
void handle_error(OPM_T *, OPM_REMOTE_T *, int, void *);

static void event_opmcon(int, User *);
static void event_proxyq();

static void cmd_scan(User *, char *);

int opm_scan_count = 0;
dlink_list proxy;



unsigned short http_ports[] = {
    8000, 8080, 3128, 80
};

unsigned short wingate_ports[] = {
    23
};

unsigned short router_ports[] = {
    23
};

unsigned short socks4_ports[] = {
    1080
};

unsigned short socks5_ports[] = {
    1080
};

unsigned short httppost_ports[] = {
    80, 8090, 3128
};

int Module_Init()
{

	int status, ch;


	status = RegisterModule(NAME,AUTHOR,VERSION,0,4,1);
	AddEvent ("USERCONN", event_opmcon);

	ch = AddCmd(s_Guardian, "SCAN", ACC_OPER, cmd_scan);

#ifdef HAVE_PTHREAD
    spawn_thread(NULL, opm_entry, NULL);
#else
		return MOD_STOP;
#endif

	return MOD_CONT;



}



void Module_Close()
{

	DelEvent("USERCON",event_opmcon);
	DelCmd(s_Guardian, "SCAN", cmd_scan);

	return;
}



static void cmd_scan(User* u, char *msg)
{

	sendto_one(s_Guardian,u,"Scanning %s",msg);
	spawn_thread((void*) msg, opm_entry, NULL);
	return;
}


static void event_proxyq()
{
	DelTimedEvent ("ProxyQue");
	return;
}

static void event_opmcon(int ac, User *u)
{

	dlink_node *dl;
	char *ip;


	if ((u->service) || (!u))
		return;

	if (sync_state != 3)
		return;

	sendto_one_notice (s_Guardian, u, "Your connection is being scanned for open proxies");

#ifdef HAVE_PTHREAD
    dl = dlink_create();
    dlink_add_tail (u, dl, &proxy);
    opm_scan_count++;
#endif

	return;
}


void opm_entry(void *args)
{
    OPM_REMOTE_T *remote;
    OPM_T *scanner;

    OPM_ERR_T err;
    int fdlimit = 1024;
    int max_read = 4096;
    int scantimeout  = 10;
    int port = 6667;

    unsigned int i, s;
    int status, ch;

    User *u;
    dlink_node *dl;

    //User *u = (User*) args;
    time_t opm_scan_timeout = 0;

    int err_code = 0;

    if (!u)
        return;
    if (!u->ip)
        return;


    scanner = opm_create();

    /* Setup callbacks */
    opm_callback(scanner, OPM_CALLBACK_OPENPROXY, &open_proxy, NULL);
    opm_callback(scanner, OPM_CALLBACK_NEGFAIL, &negotiation_failed, NULL);
    opm_callback(scanner, OPM_CALLBACK_TIMEOUT, &opm_timeout, NULL);
    opm_callback(scanner, OPM_CALLBACK_END, &end, NULL);
    opm_callback(scanner, OPM_CALLBACK_ERROR, &handle_error, NULL);


    /* Setup the scanner configuration */
    opm_config(scanner, OPM_CONFIG_FD_LIMIT, &fdlimit);
    opm_config(scanner, OPM_CONFIG_SCAN_IP, CfgSettings.local_ip);
    opm_config(scanner, OPM_CONFIG_SCAN_PORT, &port);
    opm_config(scanner, OPM_CONFIG_TIMEOUT, &scantimeout);
    opm_config(scanner, OPM_CONFIG_MAX_READ, &max_read);

    /* Setup the protocol configuration */
    for (s = ARRAY_SIZEOF(http_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_HTTP, http_ports[i]);


    for (s = ARRAY_SIZEOF(wingate_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_WINGATE, wingate_ports[i]);


    for (s = ARRAY_SIZEOF(router_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_ROUTER, router_ports[i]);


    for (s = ARRAY_SIZEOF(socks4_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_SOCKS4, socks4_ports[i]);


    for (s = ARRAY_SIZEOF(socks5_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_SOCKS5, socks5_ports[i]);


    for (s = ARRAY_SIZEOF(httppost_ports), i = 0; i < s; i++)
            opm_addtype(scanner, OPM_TYPE_HTTPPOST, httppost_ports[i]);


    //below this line is a cluster fuck ;/
    while (sync_state != SHUTDOWN)
    {
            if (opm_scan_count <= 0)
            {
                    sleep(1);
                    continue;
            }

            DLINK_FOREACH(dl,proxy.head)
            {
                u = dl->data;

                if (!(remote  = opm_remote_create(u->ip)))
                {
                    Log("Unable to create remote %s:%s",__FILE__,__LINE__);
                    dest_thread();
                    break;
                }

                err_code = opm_scan(scanner, remote);

                if (err_code != OPM_SUCCESS)
                    break;

                opm_scan_timeout = time(NULL) + 180;

                while (opm_active(scanner) > 0)
                {
                    	if (time(NULL) > opm_scan_timeout)
                        	break;

                    	opm_cycle(scanner);
                    	sleep(1);
                }

                opm_remote_free(remote);
                opm_scan_count--;

                dl = dlink_find_delete(u,&proxy);
                dlink_free(dl);

        }
    }

    opm_free(scanner);
    dest_thread();
	return;
}


void open_proxy(OPM_T *scanner, OPM_REMOTE_T *remote, int notused, void *data)
{
   USE_VAR(notused);
   USE_VAR(data);

   sendto_logchan("\002Proxy:\002 Found proxy on %s:%d", remote->ip,
         remote->port);

    return; //meh dont continue keep matching.
}


void negotiation_failed(OPM_T *scanner, OPM_REMOTE_T *remote, int notused,
      void *data)
{
    USE_VAR(scanner);
    USE_VAR(notused);
    USE_VAR(data);

    alog(DEBUG3,"OPM Negotiation failed when scanning: %s:%p",remote->ip,remote->port);
    return;
}

void opm_timeout(OPM_T *scanner, OPM_REMOTE_T *remote, int notused, void *data)
{

    USE_VAR(scanner);
    USE_VAR(notused);
    USE_VAR(data);

    alog(DEBUG3,"OPM Timeout when scanning %s:%p",remote->ip,remote->port);
	return;
}



void end(OPM_T *scanner, OPM_REMOTE_T *remote, int notused, void *data)
{

   USE_VAR(scanner);
   USE_VAR(notused);
   USE_VAR(data);

   alog(DEBUG3, "Scan on %s has ended", remote->ip);

   complete = pthread_self();
   return; //dont destroy the thread let it handle that as needed. :)
}

void handle_error(OPM_T *scanner, OPM_REMOTE_T *remote, int err, void *data)
{

   USE_VAR(scanner);
   USE_VAR(data);

   switch(err)
   {
      case OPM_ERR_MAX_READ:
            sendto_logchan("Reached MAX READ on %s:%d", remote->ip, remote->port);
            break;
      case OPM_ERR_BIND:
            break;
      case OPM_ERR_NOFD:
            sendto_logchan("Unable to allocate file descriptor for %s:%d",
               remote->ip, remote->port);
            break;
      default:
          sendto_logchan("Unknown error on %s:%d, err = %d", remote->ip,
               remote->port, err);
   }
}

