/**
 *           OMEGA SECURITY SERVICES
 *            (C) 2008 Omega Team
 *
 * Please read COPYING and README for further details.
 *
 *
 *  This module tracks sessions from an IP/HOST
 *  and limits them based on the set number to limit to.
 *  The idea behind this is to prevent a IP/HOST from haveing
 *  to many connections to the network, this can be indicative
 *  of a botnet. I am however concerned that this module
 *  has the possibility to slow omega down, since we arnt thread
 *  safe the core will need to wait for this to return, given
 *  the size of EXEMPT list will be the size of our slow down.
 *  since everything else is done with hash's and we wont take
 *  a performce hit from that... Twitch
 *
 */


#include "stdinc.h"
#include "server.h"

#define NAME "m_sessions"
#define VERSION "$Id: m_sessions.c 1380 2008-09-09 08:21:49Z twitch $"
#define AUTHOR "twitch"


/******************************************************/

//for now rip our CMD_HASH function
#define HOST_HASH(x,y) hash_safe(x,y)
#define SLIST_SIZE 1024

#define HOST_LIMIT 4
#define BUFSIZE	512

#ifdef HAVE_SQLITE

    struct sqlite3	*exempt_db;

#endif



static void event_usercon (int, User *);

static void cmd_sessions  (User *,char *);
static void cmd_exmpt     (User *,char *);

void init_session_list();

typedef struct sessions_ {

    time_t age;

    int    count;

    char   *host;

} Session;

typedef struct sesexempt_ {
    time_t age;

    time_t expires;

    char *host;

    int  count;

} SessionExempt;


dlink_list SessionsList[SLIST_SIZE];
dlink_list exemptlist;

extern int session_count;
extern size_t session_size;

static void help_sessions     (User *);
static void help_sessions_ext (User *);
static void help_sessions_ext (User *);


Session* add_session (User *);
int del_session      (User *);

int add_exempt  (char *, int);
int del_exempt  (char *);

SessionExempt *find_exempt(char *);


static void save_exemptdb (void);
static void load_exemptdb (void);

/******************************************************/

Session* add_session(User *u)
{


    dlink_node *dl;

    int index = 0;
    int found = 0;
    char *host;


    Session* s;


    if (u->service)
	    return NULL;



    if (!(host = u->host))
		return NULL;

    index = HOST_HASH(host,SLIST_SIZE);

    if (index > SLIST_SIZE)
	index = index % SLIST_SIZE;


    DLINK_FOREACH(dl,SessionsList[index].head)
    {

		s = dl->data;

		if ((!s))
			break;

		if (strcasecmp(s->host,host)==0)
		{
			found = 1;
			break;
		}

    }

    if (!found)
    {
        dl = dlink_create();
        s = (Session*) malloc(sizeof(Session));
        s->age = time(NULL);
        s->host = strdup(host);
        s->count = 0;
    }

    s->count++;

    if (!found)
	dlink_add_tail(s,dl,&SessionsList[index]);

    return s;


}

int del_session(User *u)
{

    dlink_node *dl;
    Session *s;
    char *host;


    int index = 0;
    int found = 0;


    if (!u)
        return 0;

    if (!(host = u->host))
		return 0;



    index = HOST_HASH(host,SLIST_SIZE);

    if (index > SLIST_SIZE)
        index = index % SLIST_SIZE;

    DLINK_FOREACH(dl,SessionsList[index].head)
    {

		s = dl->data;

		if ((!s))
			break;

		if ((strcasecmp(s->host,host)==0) || (strcasecmp(s->host,u->host)==0))
		{
			found = 1;
			break;
		}

     }

     if (!found)
        return found;

     if (!s)
        return 0;

     if (s->count > 0)
     	 s->count--;

     if (s->count == 0)
     {
        dl = dlink_find_delete(s,&SessionsList[index]);
        dlink_free(dl);

        if (s->host)
            free(s->host);

         free(s);
     }

     return 1;

}




/******************************************************/

int add_exempt(char *host, int count)
{
    dlink_node *dl;
    SessionExempt *se;


    if (find_exempt(host))
            return 2;

    se = (SessionExempt*) malloc(sizeof(SessionExempt));

    if (!se)
            return 1; //malloc failed

    se->host = strdup(host);
    se->count = count;
    se->age = time(NULL);

    dl = dlink_create();

    dlink_add_tail(se,dl,&exemptlist);

    return 0;

}

/********************************************************/
/**
 * del_exempt() - Delete a host from the exempt list
 *                if * is host clear all hosts from the
 *                exempt list - used on clean up :)
 *
 *  @param host - the host we are removing.
 */


int del_exempt(char *host)
{
    dlink_node *dl,*tdl;
    SessionExempt *se;
    int found = 0;

    DLINK_FOREACH_SAFE(dl,tdl,exemptlist.head)
    {

              se = dl->data;

             if ((!match_wild(se->host,host,0)) && (!match_wild(host,se->host,0)))
                    continue;

              found++;

              if (se->host)
                free(se->host);

              dl = dlink_find_delete(se,&exemptlist);
              dlink_free(dl);
              free(se);
    }

    return (found)? 1 : 0; //if we return 0 it doesnt exsist :)
}


/********************************************************/
/**
 *  find_exempt() -  find the exemption by wild matching
 *                   since we use this theory we allow
 *                   them to be added/deled in wild fashion.
 *                   and thus need to check for them in this
 *                   fashion.
 *
 *
 *  @param host - the host we are hunting for.
 *
 *
 */


SessionExempt *find_exempt(char *host)
{
    dlink_node *dl,*tdl;
    SessionExempt *se;

    DLINK_FOREACH(dl,exemptlist.head)
    {
         se = dl->data;

          if ((match_wild(se->host,host,0)))
            return  se;
    }

    return NULL;

}

/******************************************************/


static void event_usercon(int ac, User *u)
{

   Session* s;
   SessionExempt *se;


    //mother fuckers dont be callin us.
    if (ac != 1)
            return;

    //dont waist space on services hosts
    if (u->service)
            return;

    if (!(s = add_session(u)))
	return;

    if (s->count > HOST_LIMIT)
    {
	if (!(se = find_exempt(s->host)))
		sendto_logchan("\002Sessions:\002 %s has exeeded sessions limit of (%d)",s->host,HOST_LIMIT);
	else
	{
		if (s->count > se->count)
			sendto_logchan("\002Sessions:\002 %s has exeeded sessions limit of (%d)",s->host,se->count);
		else
			return;
	}
    }
    return;
}


static void event_userexit(int ac, User *u)
{
	if (!u)
		return;
	else
		del_session(u);

        return;

}

/**************************************************/

static void cmd_exempt(User *u, char *pstr)
{
        SessionExempt *se;
        dlink_node *dl,*tdl;

        int i, n, index;
        int threshold;

        int found = 0;
        char* params[512];

        n = generictoken(' ',512,pstr,params);

        if (!params[0])
        {
            sendto_one(s_Guardian,u,"Invalid parameter. You must specify an argument.");
            return;
        }

       	if (strcasecmp(params[0],"LIST")==0)
        {

            DLINK_FOREACH(dl,exemptlist.head)
            {
                se = dl->data;
                sendto_one(s_Guardian,u,"%s %d",se->host,se->count);
                found++;
            }

            if (!found)
                sendto_one(s_Guardian,u,"The session exemption list is currently empty");
            else
                sendto_one(s_Guardian,u,"Currently \002%d\002 %s.",found, (found > 1)? "entries" : "entry");

            return;

        } else if (strcasecmp(params[0],"ADD")==0) {

                if (!params[1] || !params[2])
                {
                   sendto_one(s_Guardian,u,"Invalid parameter. You must specify an argument.");
                   return;
                }

                //assume they know wtf they are doing :)
                if (((threshold = atoi(params[2])) <= HOST_LIMIT) || (threshold > 100))
                {

                       sendto_one(s_Guardian,u,"Invalid parameter. The count must be betweem %d and 100.",HOST_LIMIT);
                       return;
                }

                i = add_exempt(params[1],threshold);

                switch (i)
                {
                    case 0:
                        sendto_one(s_Guardian,u,"Set %s session limit to be %d",params[1],threshold);
                        break;
                    case 1:
                        sendto_one(s_Guardian,u,"ERROR: malloc failed while adding exempt");
                        break;
                    case 2:
                        sendto_one(s_Guardian,u,"Exemption for %s already exsists.",params[1]);
                        break;

                }


                return;

        } else if (strcasecmp(params[0],"DEL")==0) {

            if (!params[1])
            {
                   sendto_one(s_Guardian,u,"Invalid parameter. You must specify an argument.");
                   return;
            }

            i = del_exempt(params[1]);

            if (i)
                sendto_one(s_Guardian,u,"Removed exemptions matcing pattern %s",params[1]);
            else
                sendto_one(s_Guardian,u,"Session exempt for %s does not apear on the list.",params[1]);


            return;
        } else if (strcasecmp(params[0],"SAVE")==0) {

            sendto_one(s_Guardian,u,"Saving exemption databases");
            save_exemptdb ();

        } else if (strcasecmp(params[0],"LOAD")==0) {

            sendto_one(s_Guardian,u,"Force loading exemption databases");

            DLINK_FOREACH_SAFE(dl,tdl,exemptlist.head)
            {
                    se = dl->data;

                    if (se->host)
                        free(se->host);

                    dl = dlink_find_delete(se,&exemptlist);
                    dlink_free(dl);

                    if (se)
                        free(se);
            }

            load_exemptdb();
            return;

        } else {
            sendto_one(s_Guardian,u,"Invalid paramater");
            sendto_one(s_Guardian,u,"\002/msg Guardian HELP EXEMPT\002 for more information");
            return;
        }
        return;

}


static void cmd_sessions(User *u, char *pstr)
{
	int i, n;
	int threshold;

	Session *s;
	dlink_node *dl;


	char	*params[512];

	i = 0;
	n = 0;

	memset(params,0,sizeof(params));

	n = generictoken(' ',512,pstr,params);

	if (!params[0])
	{
        	sendto_one(s_Guardian,u,"Invalid parameter. You must specify an argument.");
        	return;
	}


	if (strcasecmp(params[0],"LIST")==0)
    	{
       		 if (!params[1])
            		threshold = 3;
        	 else
            		threshold = atoi(params[1]);

		if ((threshold <= 1) || (threshold > 100))
		{
			sendto_one(s_Guardian,u,"Invalid parameter, threshold must be between 1 and 100");
			return;
		}

        	do
   		{

			DLINK_FOREACH(dl,SessionsList[i].head)
			{
				s = dl->data;

				if (!s)
					break;

				if (s->count < threshold)
					continue;

				sendto_one(s_Guardian, u, "%s %d",s->host,s->count);


			}

			if (i + 1 > SLIST_SIZE)
				break;

			i++;

		} while (i <= SLIST_SIZE);

		return;

	} else if (strcasecmp(params[0], "SHOW") == 0) {
		dlink_node	*dl;
		User		*user;

		if (!params[1])
		{
			sendto_one (s_Guardian, u, "Invalid parameter, SESSION SHOW requires a hostname");
			return;
		}

		i = 0;

		DLINK_FOREACH (dl, userlist.head)
		{
			user	= (User *) dl->data;

			if ((strcasecmp (user->host, params[1])) == 0)
			{
				sendto_one (s_Guardian, u, "[%d] %s", i, user->nick);
				i++;
			}
		}

		return;
	} else {
        sendto_one(s_Guardian,u,"Invalid parameter. You must specify an argument.");
        return;
    }

}


void init_session_list()
{
        int i;
	memset(SessionsList,0,sizeof(SessionsList));

        exemptlist.head = exemptlist.tail = NULL;

	session_count = 0;
	session_size = 0;
        return;
}

/*****************************************************/
/**
 * DB functions
 */

static void save_exemptdb(void)
{
	dlink_node		*dl;
	SessionExempt   *se;
	static char		buf[BUFSIZE * 4];
	char			*errmsg;
	int				i;

#ifdef HAVE_SQLITE

	if ((i = sqlite3_exec (exempt_db, "DELETE FROM exempts", NULL, NULL, &errmsg)) == SQLITE_BUSY)
		return;

	DLINK_FOREACH (dl, exemptlist.head)
	{
		se	= (SessionExempt*) dl->data;

		if ((!se) || (!se->host) || (!se->count) || (!se->age))
			continue;

		snprintf (buf, sizeof(buf), "INSERT INTO exempts VALUES ('%s', %d, '%d')", se->host, se->count, se->age);

		if ((i = sqlite3_exec (exempt_db, buf, NULL, NULL, &errmsg)) == SQLITE_BUSY)
		{
            sendto_logchan("\002Error:\002 unable to save exempt db sqlite returned (%d)",i);
			return;
		}
	}
#endif

	return;
}


static void load_exemptdb (void)
{
	dlink_node	*dl;
	SessionExempt   *se;

	char 		**data, *errmsg, *chname;
	int		i, j, row_count, col_count, pos;


#ifdef HAVE_SQLITE

	if ((i = sqlite3_get_table (exempt_db, "SELECT * FROM exempts", &data, &row_count, &col_count, &errmsg)))
	{
		switch (i)
		{
			case SQLITE_BUSY:
				printf ("SQLite database busy\n");
				return;
		}
	}

    if (row_count == 0)
		return;

	pos	= col_count;

	for (i = 0; i < row_count; i++)
	{
	    	se	= (SessionExempt*) malloc (sizeof(SessionExempt));
            dl	= dlink_create ();

            for (j = 0; j < col_count; j++)
            {
                switch (j)
                {
                    case 0:
                        se->host  = strdup(data[pos++]);
                        break;
                    case 1:
                        se->count = atoi(data[pos++]);
                        break;
                    case 2:
                        se->age   = atoi(data[pos++]);
                        break;
                }

            }

        dlink_add_tail (se, dl, &exemptlist);
	}



#endif
    return;

}




/*****************************************************/


static void help_sessions_ext(User *u)
{
        sendto_one(s_Guardian,u,"Syntax: \002SESSION [OPTION] [PARAMS]\002");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"\002SESSION LIST [THRESHOLD]\002");
	sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"     List all currently connected hosts with sessions at least the value");
        sendto_one(s_Guardian,u,"     of THRESHOLD");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"\002SESSION SHOW [HOST]\002");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"     Show the nicks of the currently sessions for HOST");
        sendto_one(s_Guardian,u," ");
        return;
}

static void help_exempt_ext(User *u)
{

        sendto_one(s_Guardian,u,"\002Syntax: EXEMPT [ADD|DEL|LIST] [options]\002");
        sendto_one(s_Guardian,u," ");
	    sendto_one(s_Guardian,u,"\002EXEMPT LIST\002");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"     List the current sessions exemption list.");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"\002EXEMPT ADD [HOST] [COUNT]\002");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"    Allows admins to add exemptions of COUNT for a given HOST,");
        sendto_one(s_Guardian,u,"    allowing certain machines to have more sessions then the default");
        sendto_one(s_Guardian,u,"    would previously allow.");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"\002EXEMPT DEL [HOST]\002");
        sendto_one(s_Guardian,u," ");
        sendto_one(s_Guardian,u,"    Allows admins to remove exemptions from a given HOST,");
        sendto_one(s_Guardian,u,"    this will reset the connection limit for the given HOST");
        sendto_one(s_Guardian,u,"    back to the default lvalues.");
        sendto_one(s_Guardian,u,"  ");
        sendto_one(s_Guardian,u,"   Note: This does not kill off any users already connected to");
        sendto_one(s_Guardian,u,"   the network. Also if * is specified it will remove all");
        sendto_one(s_Guardian,u,"   exempts in the list.");
        sendto_one(s_Guardian,u," ");
        return;


}
static void help_sessions(User *u)
{
        sendto_one(s_Guardian,u,"SESSION - Manage and view the sessions list.");
        sendto_one(s_Guardian,u,"EXEMPT - Manipulate the sessions exempt list.");
        return;
}

/******************************************************/
/**
 * Module factories
 */

int Module_Init()
{
    int ch, status = 0;


    if ((status = RegisterModule(NAME,AUTHOR,VERSION,0,4,1)) != 0)
	return status;

    AddEvent("USERCONN",event_usercon);
    AddEvent("USEREXIT",event_userexit);

    ch = AddCmd(s_Guardian,"SESSION",ACC_ADMN,cmd_sessions);
    AddHelpExt(ch,s_Guardian,help_sessions_ext);

    ch = AddCmd(s_Guardian,"EXEMPT",ACC_ADMN,cmd_exempt);
    AddHelpExt(ch,s_Guardian,help_exempt_ext);

    AddHelp(s_Guardian,help_sessions);


    init_session_list();

#   ifdef HAVE_SQLITE


	if (sqlite3_open ("exemptdb", &exempt_db))
		return MOD_CONT;

    load_exemptdb ();


    AddTimedEvent("SAVEDB EXEMPT",save_exemptdb,1800);

#   endif

    return MOD_CONT;
}


void Module_Close()
{

    dlink_node *dl,*tdl;
    SessionExempt *se;
    Session *s;

    int i = 0;


	Log("[%s] unloading module",NAME);


    DelEvent("USERCONN",event_usercon);
    DelEvent("USEREXIT",event_userexit);


#ifdef HAVE_SQLITE

    DelTimedEvent ("SAVEDB EXEMPT");

    if (exempt_db)
		sqlite3_close (exempt_db);

#endif



    del_exempt("*"); //clear our exempt list.

    do
    {

	DLINK_FOREACH_SAFE(dl,tdl,SessionsList[i].head)
	{
		s = dl->data;
		if (!s)
			break;

		if (s->host)
			free(s->host);

		dl = dlink_find_delete(s,&SessionsList[i]);
		dlink_free(dl);
		free(s);
	}

	i++;

    } while (i <= SLIST_SIZE);


    init_session_list();

    DelCmd (s_Guardian,"SESSION",cmd_sessions);
    DelCmd (s_Guardian,"EXEMPT", cmd_exempt);


    return;
}


