/**
 *
 *         OMEGA SECURITY SERVICES
 *     (C) 2008 Omega Development Team
 *
 *   For COPYING instructions please read
 *   the LICENSE file.
 *
 *   Holds Client command hash functions.
 *   This is for client commands so clients
 *   do not need to wade through there own PM's
 */



#include "stdinc.h"
#include "server.h"



const unsigned int prime_table_length = sizeof(primes)/sizeof(primes[0]);
const float max_load_factor = 0.65;

extern Module* currentModule;


int hc = 0;

void HelpNull(User*);


/******************************************************/

int AddCmd(User* u,char* name,uint32 access,void (*fnct)(User*,char*))
{

        dlink_node *dl = dlink_create();
	CmdHash *c = (CmdHash*) malloc(sizeof(CmdHash));

	int index = HASH(name,1024);

	c->name = strdup(name);

	c->access = access;
	c->fnct = fnct;
	c->hascmd = 1;
	c->hashlp = 0;
	c->hashlpext = 0;

       alog(DEBUG3,"Adding CLIENT command %s (%d) (Location: %p)",name,index,fnct);

       dlink_add_tail(c,dl,&u->cmd_list[index]);

	return index;


}


/*****************************************************/
int AddCmdHelp(int index,User* u, void (*hlpcmd)(User*), void (*help_cmd_ext)(User*))
{
	return 0;
}


int AddHelp(User* u, void (*helpcmd)(User*))
{

    HelpEntry *ch,*ch2;
    dlink_node *dl;


    alog(DEBUG3,"Adding help (Location: %p)",helpcmd);

    if (!u)
	return -1;

   dl = dlink_create();

   ch = (HelpEntry*) malloc(sizeof(HelpEntry));

   ch->routine = helpcmd;

   dlink_add_tail(ch,dl,&u->cmd_help);

   return 1;


}

int AddHelpExt(int index,User* u, void (*helpcmd)(User*))
{

    HelpEntry *hc; 
    dlink_node *dl;

    if (!index)
        return -1;

    dl = dlink_create();

    hc = (HelpEntry*) malloc(sizeof(HelpEntry));

    hc->routine = helpcmd;

    dlink_add_tail(hc,dl,&u->help_ext_list[index]);

    alog(DEBUG3,"Adding EXTENDED HELP For command index (%d) (Location: %p)",index,helpcmd);

    return 0;
}


int DelHelp(User *u, void (*helpcmd)(User*))
{

	HelpEntry* he;
	dlink_node *dl;

	DLINK_FOREACH(dl,u->cmd_help.head)
	{
		he = dl->data;

		if (he->routine != helpcmd)
			continue;
		
		dl = dlink_find_delete(he,&u->cmd_help);
		dlink_free(dl);

		if (he)
			free(he);

		break;

	

	}
	return;
}

/******************************************************/


void DelCmd(User* u, char* cmd, void (*fnct)(User*,char*))
{
    dlink_node *dl,*tdl;
    CmdHash *ch;


    int index = HASH(cmd,1024);

    DLINK_FOREACH_SAFE(dl,tdl,u->cmd_list[index].head)
    {
            ch = dl->data;

      		if (ch->fnct != fnct)
			continue;


                dl = dlink_find_delete(ch,&u->cmd_list[index]);
                dlink_free(dl);

                free(ch);

		break;
    }

    return;
}


/*****************************************************/
/**
 * int do_cmd() - Handles a command to a client -
 *                do all the santatity checks here.
 *
 *  @param cmd         = The command we are actioning on.
 *  @param us          = A pointer to OUR client struct
 *  @param u           = A pointer to the SRC of the command
 *  @param ac          = The argument count being passed into the command
 *                       handler - used to see if av is null :)
 *  @param av          = The PARAMATERS to the command - the original
 *                       command.
 *  @return int
 *             0 = Success
 *             1 = No cmd for it was found.
 *             2 = Access Denied.
 *             3 = Fail something was passed wrong
 */



int do_cmd(char *cmd, User* us, User *u, int ac, char *av)
{
 	   CmdHash* c;
	   HelpEntry *he;
           dlink_node *dl;

	   int index = 0;
	   int found = 0;

 	   if (!us || !u)
 	     	return 0;


	   if (nomodules)
	   {
		sendto_one(us,u,"Server is currently in skeleton mode - no commands are available");
		return 0;
	   }

	   if (strcasecmp(cmd,"HELP")==0)
	   {
           	 if (ac == 2)
            	 {
                    	  index = HASH(av,1024);

	   	    	  DLINK_FOREACH(dl,us->cmd_list[index].head)
			  {
         				c = dl->data;
			    		break;
			  }

			  if (!c)
			  {
                  sendto_one (us, u, "No such command");
                  return 1;
              }

			  if (!check_access(u,c->access))
			  {
					sendto_one(us,u,"You do not have the correct permissions to view the help for this command.");
					return 1;
			  }


			  DLINK_FOREACH(dl,us->help_ext_list[index].head)
			  {
					he = dl->data;
                          		he->routine(u);
					found = 1;
			  }

			  sendto_one(us,u,"Limited to access level (%lu)",c->access);

			
		    if (found)
			return 0;
		    else
                    	sendto_one(us,u,"There is no help available for that command.");

                    return 0;
            	}

            HELP_FOREACH(us,u);
            return 0;

	   }

	   index = HASH(cmd,1024);

	   DLINK_FOREACH(dl,us->cmd_list[index].head)
	   {

         	c = dl->data;


          	if (strcasecmp(c->name,cmd)==0)
          	{
                       if (!check_access(u, c->access))
		       {
                           ACCESS_DENIED(us,u);
			   return 1;
			}

                	if (!c->hascmd)
                        	continue;

                   	 assert(c->hascmd == 1);

                   	 c->fnct(u,av);
                   	 found = 1;
          	}

	   }

       return (found)? 0 : 1;
}



/*****************************************************/

int HELP_FOREACH(User* us, User *u)
{
      HelpEntry* he;
      dlink_node *dl;

	DLINK_FOREACH(dl,us->cmd_help.head)
	{
		he = dl->data;
		alog(DEBUG3,"Calling HELP command at location: %p",he->routine);
		he->routine(u);
	}

      return 0;
}



/*****************************************************/


void ACCESS_DENIED(User* s,User *u)
{
		sendto_one(s,u,"Permission Denied.");
		return;
}


char *CmdErr(int err)
{
	const char *cmd_err_codes[] = {
		"Command success, no error.",
		"Invalid command - No such command found.",
		"Permission Denied.",
		"Command Error, Invalid paramaters.",
		"Command Error, Invalid index.",
		"Command Error, HASH does not exsist."
	};

	return (char*) cmd_err_codes[err];
}

/*****************************************************/


void HelpNull(User *u)
{
        return;
}
