#include "stdinc.h"
#include "configparse.h"
#include "server.h"
 

#define PARAM_ERROR(ce, s)  { fprintf (stderr, "%s:%i: no parameter for %s " \
                                    "config option: %s\n", (ce)->ce_fileptr->cf_filename, \
                                    (ce)->ce_varlinenum, (s), (ce)->ce_varname); \
                               alog (3, "%s:%i: no parameter for %s " \
                                    "config option: %s\n", (ce)->ce_fileptr->cf_filename, \
                                    (ce)->ce_varlinenum, (s), (ce)->ce_varname); \
                               return 1; }


#define INVALID_PARAM(ce, s)  { fprintf (stderr, "%s:%i: Invalid parameter %s " \
                                    "config option: %s\n", (ce)->ce_fileptr->cf_filename, \
                                    (ce)->ce_varlinenum, (s), (ce)->ce_varname); \
                               alog (3, "%s:%i: Invalid parameter %s " \
                                    "config option: %s\n", (ce)->ce_fileptr->cf_filename, \
                                    (ce)->ce_varlinenum, (s), (ce)->ce_varname); }



#define FOREACH_PARAM(ce) for (ce = ce->ce_entries; ce; ce = ce->ce_next)

int modcnt = 0;

struct ConfTable
{
    char    *name;
    int     rehashable;
    int     (*handler)();
};

static int  generic_subblock_handler (CONFIGENTRY *, char *, struct ConfTable *);



static int  c_serverinfo        (CONFIGENTRY *);
static int  c_serverinfo_name   (CONFIGENTRY *);
static int  c_serverinfo_port   (CONFIGENTRY *);
static int  c_serverinfo_uplink (CONFIGENTRY *);
static int  c_serverinfo_desc   (CONFIGENTRY *);
static int  c_serverinfo_pass   (CONFIGENTRY *);
static int  c_serverinfo_num    (CONFIGENTRY *);
static int	c_serverinfo_sid	(CONFIGENTRY *);
static int	c_serverinfo_vhost	(CONFIGENTRY *);
static int	c_serverinfo_network	(CONFIGENTRY *);

static int  c_protocol          (CONFIGENTRY *);
static int  c_protocol_name     (CONFIGENTRY *);
static int  c_protocol_ts6      (CONFIGENTRY *);


static int  c_settings				(CONFIGENTRY *);
static int  c_settings_logchan		(CONFIGENTRY *);
static int  c_settings_privmsg		(CONFIGENTRY *);
static int	c_settings_localip		(CONFIGENTRY *);
static int	c_settings_pidfile		(CONFIGENTRY *);
static int	c_settings_logusers		(CONFIGENTRY *);

static int   c_guardian              (CONFIGENTRY *);
static int   c_guardian_nick         (CONFIGENTRY *);
static int   c_guardian_user         (CONFIGENTRY *);
static int   c_guardian_host         (CONFIGENTRY *);
static int   c_guardian_desc         (CONFIGENTRY *);
static int   c_guardian_channels     (CONFIGENTRY *);
static int   c_gmodules              (CONFIGENTRY *);

static int c_access         (CONFIGENTRY *);
static int c_access_flags   (CONFIGENTRY *);
static int c_access_host    (CONFIGENTRY *);


Access *CurrentAccess = NULL;

static struct ConfTable conf_root_table[] =
{
    { "SERVERINFO", 1, c_serverinfo },
    { "PROTOCOL", 1, c_protocol },
    { "SETTINGS", 1 , c_settings },
    { "GUARDIAN", 1 , c_guardian },
    { "ACCESS", 1, c_access },
    { NULL, 0, NULL }
};

static struct ConfTable conf_protocol_table[] =
{
    { "NAME",   0,  c_protocol_name },
    { "TS6",	0,	c_protocol_ts6 },
    { NULL,     0,  NULL }
};

static struct ConfTable conf_access_table[] =
{
	{ "FLAGS",	0,	c_access_flags },
	{ "HOSTS",  0,	c_access_host },
	{ NULL,     0,	NULL }
};

static struct ConfTable conf_settings_table[] =
{
    { "LOGCHAN",	0,  c_settings_logchan },
    { "PRIVMSG",	1,  c_settings_privmsg },
    { "LOCALIP",	1,  c_settings_localip },
    { "LOGUSERS",	0,  c_settings_logusers },
	{ "PIDFILE",	0,	c_settings_pidfile },
    { NULL,     0,  NULL }
};

static struct ConfTable conf_guardian_table[] =
{
        { "NICK",               0,      c_guardian_nick },
        { "USER",               0,      c_guardian_user },
        { "HOST",               0,      c_guardian_host },
        { "DESC",               0,      c_guardian_desc },
        { "CHANNELS",           0,      c_guardian_channels },
	 { "MODULES",            1,      c_gmodules },
        { NULL,                 0,      NULL }
};

static struct ConfTable conf_serverinfo_table[] =
{
    { "NAME",   0,  c_serverinfo_name },
    { "PORT",   0,  c_serverinfo_port },
    { "UPLINK", 0,  c_serverinfo_uplink },
    { "DESC",   1,  c_serverinfo_desc },
    { "PASS",   1,  c_serverinfo_pass },
    { "NUMERIC", 0, c_serverinfo_num },
    { "SID",	   0,	c_serverinfo_sid },
    { "VHOST",   0,	c_serverinfo_vhost },
	{ "NETWORK",	0,	c_serverinfo_network },
    { NULL,      0,  NULL }
};


void InitDefaults ()
{

    strcpy (CfgSettings.servername, "security.network.tld");
    strcpy (CfgSettings.uplink,		"hub.network.tld");
    strcpy (CfgSettings.pass,		"password");
    strcpy (CfgSettings.desc,		"Security Services");
    strcpy (CfgSettings.numeric,	"86");
    strcpy (CfgSettings.sid,		"666");
	strcpy (CfgSettings.network,	"Misconfigured");

    CfgSettings.port    = 6667;

	CfgSettings.vhost[0]	= '\0';
//	*CfgSettings.vhost = '\0';

	CfgSettings.local_ip[0]	= '\0';

	snprintf (CfgSettings.pidfile, sizeof(CfgSettings.pidfile), "%s/omega.pid", DPATH);

    strcpy (logchan, "#Services");
    strcpy (guardian.nick,	"Guardian");
    strcpy (guardian.host,	"network.tld");
    strcpy (guardian.user,	"security");
    strcpy (guardian.desc,	"Network Security Service");

    CfgSettings.privmsg	= 0;
    memset(guardian_modules,0,sizeof(guardian_modules));
    gmod_cnt = 0;

}

void VerifyConf ()
{
        
	if (CfgSettings.local_host != NULL)
		free (CfgSettings.local_host);

	if (CfgSettings.local_ip[0] == '\0')
		CfgSettings.local_host = get_hostname (NULL);
	else
		CfgSettings.local_host = get_hostname (CfgSettings.local_ip);
}

void ConfLoad (void)
{
    CONFIGFILE          *cfptr;
    CONFIGFILE          *cfp;
    CONFIGENTRY         *ce;
    struct ConfTable    *ct = NULL;

    cfptr = cfp = config_load (CfgSettings.conf_name);

    if (cfp == NULL)
    {
        fprintf (stderr, "Unable to open configuration file (%s): %s\n",
                    CONFIG_NAME, strerror(errno));
        exit (0);
    }

    for (;cfptr;cfptr = cfptr->cf_next)
    {
        for (ce = cfptr->cf_entries;ce;ce = ce->ce_next)
        {
            for (ct = conf_root_table; ct->name; ct++)
            {
                if (!strcasecmp (ct->name, ce->ce_varname))
                {
                    ct->handler (ce);
                    break;
                }
            }

            if (ct->name == NULL)
            {
                fprintf (stderr, "%s:%i invalid config option :%s\n", ce->ce_fileptr->cf_filename,
                        ce->ce_varlinenum, ce->ce_varname);
            }
        }
    }

    config_free (cfp);
}

void ConfReload (void)
{
  CONFIGFILE *cfptr;
  CONFIGFILE *cfp;
  CONFIGENTRY *ce;
  struct ConfTable *ct = NULL;

  cfptr = cfp = config_load(CfgSettings.conf_name);
  if (cfp == NULL)
  {
    fprintf (stderr, "Unable to open configuration file (%s): %s\n",
         CONFIG_NAME, strerror(errno));
  }
  for (;cfptr;cfptr = cfptr->cf_next) /* Handle include files? */
  {
    for (ce = cfptr->cf_entries;ce;ce = ce->ce_next)
    {
      for (ct = conf_root_table; ct->name; ct++)
      {
        if (!strcasecmp(ct->name, ce->ce_varname))
        {
          if (ct->rehashable == 1)
            ct->handler(ce);
          break;
        }
      }
      if (ct->name == NULL)
      {
		fprintf (stderr, "%s:%i invalid config option: %s\n", ce->ce_fileptr->cf_filename, ce->ce_varlinenum,
				ce->ce_varname);
      }
    }
  }
  config_free(cfp);
}

static int generic_subblock_handler(CONFIGENTRY *ce, char *subblock,
                                    struct ConfTable *table)
{
  struct ConfTable *ct = NULL;

  for (ce = ce->ce_entries;ce;ce = ce->ce_next)
  {
    for (ct = table; ct->name; ct++)
    {
        if (!strcasecmp(ct->name, ce->ce_varname))
        {
            ct->handler(ce);
            break;
        }
    }
    if (ct->name == NULL)
    {
      fprintf (stderr, "%s:%i invalid %s config option: %s\n", ce->ce_fileptr->cf_filename,
            ce->ce_varlinenum, subblock, ce->ce_varname);
    }
  }
  return 0;
}

/***************************************************/

static int c_gmodules (CONFIGENTRY *ce)
{


     gmod_cnt = 0;
     memset(guardian_modules,0,sizeof(guardian_modules));

     FOREACH_PARAM(ce)
     {

		
		strlcpy(guardian_modules[gmod_cnt],ce->ce_varname,sizeof(guardian_modules[gmod_cnt]));
		if (!strstr(guardian_modules[gmod_cnt],".so"))
			strlcat(guardian_modules[gmod_cnt],".so",sizeof(guardian_modules[gmod_cnt]));
		gmod_cnt++; 
     }

    return 0;
}


static int c_access (CONFIGENTRY *ce)
{
     dlink_node	*dl;

   
     FOREACH_PARAM(ce)
     {
		if (!ce->ce_varname)
			continue;

		CurrentAccess	= (Access*) malloc(sizeof(Access));
        dl 				= dlink_create();

		strlcpy (CurrentAccess->name, ce->ce_varname, sizeof(CurrentAccess->name));

		memset (CurrentAccess->cname, '\0', sizeof(CurrentAccess->cname));

		CurrentAccess->flags		= 0x0;
		CurrentAccess->hosts.head	= CurrentAccess->hosts.tail	= NULL;

		generic_subblock_handler (ce, "ACCESS", conf_access_table);
        dlink_add_tail (CurrentAccess, dl, &accesslist);
    }


    return 0;

}


static int c_access_host (CONFIGENTRY *ce)
{
	char *tmp_host;
	dlink_node *dl;

	FOREACH_PARAM(ce)
	{	
		dl = dlink_create();
		tmp_host = strdup(ce->ce_varname);
		dlink_add_tail (tmp_host, dl, &CurrentAccess->hosts);

	}
	return 0;
}

static int c_access_flags (CONFIGENTRY *ce)
{
	int i = 0; 
        int found = 0;
      	
	CurrentAccess->flags = 0x0000000;
	
	FOREACH_PARAM(ce) 
	{
	
		i = 0;
		found = 0;

		if (ce->ce_varname)
 		{
			
			for ( ; CfgAccess[i].name && CfgAccess[i].flags; i++)
			{
				if (compair(ce->ce_varname,CfgAccess[i].name)==0)
				{
					found = 1;
					if (CurrentAccess->flags)
						CurrentAccess->flags = CfgAccess[i].flags;
					else
						CurrentAccess->flags |= CfgAccess[i].flags;
					break;
				}
			}

			if (!found)
				fprintf(stderr,"Invalid Parameter in Access::Flags %s\n",ce->ce_varname);

		}

		else
			break;
	}
	

	return 0;	
}


/***************************************************/
static int c_serverinfo (CONFIGENTRY *ce)
{
    generic_subblock_handler (ce, "SERVERINFO", conf_serverinfo_table);
    return 0;
}



static int c_serverinfo_num (CONFIGENTRY *ce)
{
   if (ce->ce_vardata == NULL)
   {
        PARAM_ERROR (ce,"serverinfo");
   }

   strlcpy(CfgSettings.numeric,ce->ce_vardata, sizeof(CfgSettings.numeric));
   return 0;
}

static int c_serverinfo_name (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "serverinfo");
    }

    strlcpy (CfgSettings.servername, ce->ce_vardata, sizeof(CfgSettings.servername));
    return 0;
}

static int c_serverinfo_port (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "serverinfo");
    }

    CfgSettings.port    = ce->ce_vardatanum;
    return 0;
}

static int c_serverinfo_uplink (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "serverinfo");
    }

    /* Due to a bug I found awhile back, where IPv6 can be enclapsed in [], we must account for this */

    if (ce->ce_vardata[0] == '[')
    {
        int len;
        ce->ce_vardata++;
        len = strlen(ce->ce_vardata);

        if ((len > 0) && (ce->ce_vardata[len - 1] == ']'))
            ce->ce_vardata[len - 1] = '\0';

        fprintf (stderr, "%s:%i WARNING: Brackets [] used in serverinfo uplink", ce->ce_fileptr->cf_filename,
                ce->ce_varlinenum);
    }

    strlcpy (CfgSettings.uplink, ce->ce_vardata, sizeof(CfgSettings.uplink));
    return 0;
}

static int c_serverinfo_pass (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "serverinfo");
    }

    strlcpy (CfgSettings.pass, ce->ce_vardata, sizeof(CfgSettings.pass));
    return 0;
}

static int c_serverinfo_desc (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "serverinfo");
    }

    strlcpy (CfgSettings.desc, ce->ce_vardata, sizeof(CfgSettings.desc));
    return 0;
}

static int c_serverinfo_sid (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "serverinfo");
	}

	strlcpy (CfgSettings.sid, ce->ce_vardata, sizeof(CfgSettings.sid));
	return 0;
}

static int c_serverinfo_vhost (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "serverinfo");
	}

	if (ce->ce_vardata[0] == '[')
	{
		int	len;
		ce->ce_vardata++;

		len	= strlen(ce->ce_vardata);

		if ((len > 0) && (ce->ce_vardata[len - 1] == ']'))
			ce->ce_vardata[len - 1] = '\0';
	}

	strlcpy (CfgSettings.vhost, ce->ce_vardata, sizeof(CfgSettings.vhost));

	return 0;
}

static int c_serverinfo_network (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "serverinfo");
	}

	strlcpy (CfgSettings.network, ce->ce_vardata, sizeof(CfgSettings.network));
	return 0;
}

/*********************************************/

static int c_protocol (CONFIGENTRY *ce)
{
    generic_subblock_handler (ce, "PROTOCOL", conf_protocol_table);
    return 0;
}


static int c_protocol_name (CONFIGENTRY *ce)
{


    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "protocol");
    }

    strlcpy (CfgSettings.protocol, ce->ce_vardata, sizeof(CfgSettings.protocol));
     return 0;
}

static int c_protocol_ts6 (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "protocol");
	}

	IRCd.ts6	= ce->ce_vardatanum;
	return 0;
}

/*******************************************/

static int c_settings (CONFIGENTRY *ce)
{
    logclients = 0; //default this to 0 so if were rehashed
                    //and its commented we will know :)

    generic_subblock_handler (ce, "SETTINGS", conf_settings_table);
    return 0;
}


static int c_settings_logchan (CONFIGENTRY *ce)
{

    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "settings");
    }

    strlcpy (logchan, ce->ce_vardata, sizeof(logchan));
    return 0;
}

static int c_settings_privmsg (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "settings");
    }

    CfgSettings.privmsg = ce->ce_vardatanum;
    return 0;
}

static int c_settings_localip (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "settings");
	}

	strlcpy (CfgSettings.local_ip, ce->ce_vardata, sizeof(CfgSettings.local_ip));
	return 0;
}

static int c_settings_pidfile (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "settings");
	}

	snprintf (CfgSettings.pidfile, sizeof(CfgSettings.pidfile), "%s/%s", DPATH, ce->ce_vardata);
	return 0;
}

static int c_settings_logusers (CONFIGENTRY *ce)
{
	if (ce->ce_vardata == NULL)
	{
		PARAM_ERROR (ce, "settings");
	}

	logclients	= ce->ce_vardatanum;
	return 0;
}

/*******************************************/


static int c_guardian (CONFIGENTRY *ce)
{
    generic_subblock_handler (ce, "GUARDIAN", conf_guardian_table);
    return 0;
}

static int c_guardian_nick (CONFIGENTRY *ce)
{
        if (ce->ce_vardata == NULL)
        {
                PARAM_ERROR (ce, "guardian");
        }

    strlcpy (guardian.nick, ce->ce_vardata, sizeof(guardian.nick));
        return 0;
}


static int c_guardian_user (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "guardian");
    }

    strlcpy (guardian.user, ce->ce_vardata, sizeof(guardian.user));
    return 0;
}

static int c_guardian_host (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "guardian");
    }

    strlcpy (guardian.host, ce->ce_vardata, sizeof(guardian.host));
    return 0;
}

static int c_guardian_desc (CONFIGENTRY *ce)
{
    if (ce->ce_vardata == NULL)
    {
        PARAM_ERROR (ce, "guardian");
    }

    strlcpy (guardian.desc, ce->ce_vardata, sizeof(guardian.desc));
    return 0;
}


static int c_guardian_channels (CONFIGENTRY *ce)
{
	char	*chan;
	dlink_node	*dl;

        if (ce->ce_varname == NULL)
        {
                PARAM_ERROR (ce, "guardian");
        }

        for (ce = ce->ce_entries;ce;ce = ce->ce_next)
        {
			chan	= malloc(sizeof(chan));
			dl		= dlink_create ();

			chan	= strdup(ce->ce_varname);
			dlink_add	(chan, dl, &guardian.chanlist);
        }

        return 0;
}

