#include "stdinc.h"
#include "core.h"
#include "log.h"
#include "user.h"
#include "channel.h"
#include "protocol.h"

char *LogType[] = {
	"Info",     //0
	"Debug",    //1
	"Debug 2",  //2
	"Error",    //3
	"Critical", //4
	"Event",    //5
	"Link",     //6
	"LogUser",  //7
	"LogChan",  //8
	"LogIO",    //9
	"Module",   //10
	"Debug3",   //11
	0
};

static unsigned long curday = 0;

void GetLog(void);

static void CleanUpLogs(int,char**);


/*************************************************/
/** 
  * Init Function - Called when server starts.
  */
  
  
void InitLog(void)
{
	AddEvent("CLEANUP",CleanUpLogs);
	GetLog();
}


/*************************************************/
void GetLog(void) 
{
        char LogName[100];
	char timestamp[32];
	
	 time_t t;
         struct tm *tmp;

         time(&t);
        tmp = localtime(&t);


	#ifndef HAVE_PTHREAD
		if (logfd > 0 || logfd)
		{
			if (tmp->tm_mday == curday)	
				return;
			else
				fclose(logfd);
		} 
			
	
	
		snprintf(LogName, sizeof(LogName), "%s%s%d%d%d",DPATH,LPATH,tmp->tm_year+ 1900,tmp->tm_mon,tmp->tm_mday);
		curday = tmp->tm_mday;	
	#else

		snprintf(LogName, sizeof(LogName), "%s%s%lu",DPATH,LPATH,(unsigned long) getpid());
	#endif

        if (!logfd || logfd <= 0)
        {
      		logfd = fopen(LogName,"a");
		fprintf(logfd,"-----------------------------------------------\n");
      		fprintf(logfd,"Session Start on %d/%d/%d at %lu \n",tmp->tm_mon,tmp->tm_mday,tmp->tm_year+ 1900,(unsigned long) time(NULL));
     		fprintf(logfd,"-----------------------------------------------\n");
        }
      
      return;	

}

/*************************************************/

void alog(int TYPE, char *fmt, ...)
{ 

    char logstr[1025];
	va_list		args;
	
	GetLog();
	
	va_start(args,fmt);	
	vsnprintf(logstr,1024,fmt,args);

	//Event ("LOG", 1, &evhinfo);

	//move to using our debug level now :) 
	if (TYPE == 1 && !debug)
			return;

	if (TYPE == 2)
	{
		if ((debug == 2) || (debug == 3))
			/*  */ ;
		else
			return;

	}

	if (TYPE == DEBUG3 && debug != 3)
		return;


	if ((logfd) && (logfd > 0))
    {
		
		fprintf(logfd,"[%s] %s\n",LogType[TYPE],logstr);
        fflush(logfd);
    }

	if (debug && nofork)
 		printf("[%s] %s\n",LogType[TYPE],logstr);

	va_end(args);
	
	memset(logstr,0,sizeof(logstr));
	
	//free(logstr);
	//logstr = NULL;
	
	return;
}

/***********************************/

void CloseLog(void) 
{
	fclose(logfd);
}

/***********************************/
void sendto_logchan (char *fmt, ...)
{
    va_list     args;
    char        *tmp;

    //no need to assign any memory logchan isnt on
    if (uselogchan != 1)
		return;

    tmp = malloc(sizeof(char) * 1024);

	va_start (args, fmt);
   	vsnprintf (tmp, 1024, fmt, args);

    if (!s_Guardian)
        return;

    if ((sync_state == 3) && logchan) 
	  	send_line (":%s PRIVMSG %s :%s", (IRCd.ts6)? s_Guardian->uid : s_Guardian->nick, logchan, tmp);
   
    va_end(args);
    memset(tmp,0,sizeof(tmp));

    free (tmp);
    return;
}


/************************************/

void sendto_console(char *fmt, ...)
{
     va_list     args;
     char        *tmp;

     tmp = malloc(sizeof(char) * 1024);

     va_start (args, fmt);
     vsnprintf (tmp, 1024, fmt, args);

     fprintf(stderr,"%s\r\n", tmp);
 
    va_end(args);
    memset(tmp,0,sizeof(tmp));

    free (tmp);
    return;
              
}

/************************************************/
 /**
 *  Log - This function logs to our log file, as
 *        well as sends to logchan... This doesnt 
 *        do any implicit checks, and will rely on
 *        the user to know what he/she is doing.
 *
 *  @param TYPE - The log level/type 
 *  @param fmt  - Formatted VA_ARG strings, or 
 *                standard parameters.
 *  @return void
 */


void Log(char *fmt, ...)
{
     va_list     args;
     char        *tmp;

     tmp = malloc(sizeof(char) * 1024);

     va_start (args, fmt);
     vsnprintf (tmp, 1024, fmt, args);


    sendto_logchan(tmp); //send to our logchan
    
    alog(0,tmp);      //then alog 
    
 
    va_end(args);
    memset(tmp,0,sizeof(tmp));

    free (tmp);
    return;
              
}

/************************************/

static void CleanUpLogs(int argc,char **argv)
{

	alog(1,"Cleaning up log files");
	
	if (logfd)
	{
		fprintf(logfd,"-----------------------------------------------\n");
		fprintf(logfd,"Sesson Ended %lu\n",time(NULL));
		fprintf(logfd,"-----------------------------------------------\n");
		
		fflush(logfd);
		fclose(logfd);
	}

}
