#include "stdinc.h"
#include "server.h"
#include "parser.h"

static inline int string_to_array(char *string, const char **parv);
static void HandleClientCmds(User* u, int ac, char** av);


void parse(char *arv);

/**********************************************/
/**
  * Hook our socket so we can handle our own socket events
  */

int psr_init()
{
    AddUserCmd("PRIVMSG", HandleClientCmds);
	return 0;
}

/**********************************************/

static inline int
string_to_array(char *string, const char **parv)
{

    char *p;
	char *buf = string;
	int x = 1;


	//meh how did this happen?
	if ((!buf) || (!string))
		return 0;

	if(*buf == '\0')
		return x;

	do
	{
		if(*buf == ':')	/* Last parameter */
		{
			buf++;
			parv[x++] = buf;
			parv[x] = NULL;
			return x;
		}
		else
		{
			parv[x++] = buf;
			parv[x] = NULL;
			if((p = strchr(buf, ' ')) != NULL)
			{
				*p++ = '\0';
				buf = p;
			}
			else
				return x;
		}
		while (*buf == ' ')
			buf++;
		if(*buf == '\0')
			return x;
	}
	/* we can go upto parv[MAXPARA], as parv[0] is taken by source */
	while (x < MAXPARA);

	if(*p == ':')
		p++;

	parv[x++] = p;
	parv[x] = NULL;
	return x;
}

/**********************************************/


void parse(char *inbuf)
{

        Link* li;
        User* u;

	const char *para[MAXPARA + 2];

	char *command = NULL;
        char *s = NULL;
	char *pos = NULL;

	int n = 0;

	memset(para,0,sizeof(para));

	if ((!inbuf) || (*inbuf == '\0'))
		return;

	alog(2,"RECV: %s", inbuf);

	for (n = 0; n < MAXPARA; n++)
		para[n] = NULL;

	if ((pos = strchr (inbuf, ' ')))
	{
		*pos = '\0';
		pos++;

		if (*inbuf == ':')
		{
			inbuf++;
			para[0]	= inbuf;

			if ((s = strchr(pos, ' ')))
			{
				*s = '\0';
				s++;

				command = pos;
			}
			else
			{
				s = NULL;
				command = pos;
			}
		}
		else
		{
			s = pos;
			command = inbuf;
		}
	}

	if (s)
		n = string_to_array (s, para);

	/* assert for safety, if command is NULL abort */
	s_assert(command != NULL);

       //okay heres a few things we need to keep in mind
       //PARA[0] is not always a server s<>s allows for it
       //to be null if CMD is directly from the uplink server

       //always assume this is from the uplink
	if (!para[0])
		HandleServCmd (command, NULL, n, (char **)para);

	else
	{
		if ((li = find_serv((char*)para[0])))
		{
			HandleServCmd (command, li, n, (char **)para); //handles a commad from the server
			return;
		}

		if ((u = find_user((char*)para[0])))
        {
			u->lastsent = time(NULL); //set our time so we can have accurate WHOIS
			HandleUserCmd(command, u, n, (char**)para);

			return;
		}

		if (n > 2)
			if (((strcasecmp(command, "NOTICE")) == 0) && ((strcasecmp (para[1], "AUTH")) == 0))
				return;

		fprintf (stderr, "Unknown prefix: %s\n", para[0]);
		return;

	}

	return;
}

/**********************************************/
/**
  * break_params()
  * Breaks argv up starting starting at start and ending at break
  * break can be argc it will hault auto if it is :)
  */


static inline int break_params(int argc, char **argv,const char **para,int start,int brk)
{
	int i = 1;
       int cnt = 0;

	memset(para,0,sizeof(para));
	while (i <= argc)
	{
		if (i >= start)
		{
			para[cnt] = (char*) argv[i];
			cnt++;
		}
		if (i >= brk)
			break;
		i++;
	}
	return 0;
}


/**********************************************/

void psr_HandleEvent(Socket* s,int et)
{
    char	*line;

	if (!(s->flags & SOCK_UPLINK))
		return;

	switch (et)
	{
		case EVENT_READ:
				while (1)
				{
					line = GetLine (servsock);

					if (line == NULL)
						break;
					parse (line);

                    memcounts.buflinesize   -= sizeof(line);
                    memcounts.buflinecnt--;
                    free (line);
				}
		break;
	}

}


static void HandleClientCmds(User* u, int ac, char** av)
{

   	User* us;

	char *params[512];
	char param[512];
	CmdHash *c = NULL;
	Channel *ch;
	int n;
	int x = 0;
	int index = 0;
	int rc = 0;

	int i = 0;

    for (i = 0; i <= 512; i++)
        params[i] = 0;

	if (!av[2])
		return;

	if (!u)
	{
		sendto_logchan ("Unknown user");
		return;
	}

   	//really we should only be getting commands
	//from Users directly for now.
       if (*av[1] == '#')
		return;

	if (*av[2] == '\001' || *av[2] == '\01' || *av[2] == '\1')
		return;

	//Server will only recieve PM's from users that are connected
	//directly on it as per RFC.
    if (!(us = find_user(av[1])))
            return;



	if ((ch = find_channel(logchan)))
		sendto_channel(us,ch,"%s: %s",u->nick,av[2]);


        n = generictoken(' ',512,av[2],params);

	if (!params)
		return;

	ExpandParv(param,512,1,n,params);
	x = do_cmd(params[0], us, u, n, param);

	/**
     * dont waist our logfiles on invalid command just kick
	 * back that it was invalid... and return off
     */

	if (x == 1 && rc == 0)
	{
		sendto_one(us,u,"%s",CmdErr(x));
		return;
	}

       if (x > 1)
	{
	   alog(2,"Error when processing do_cmd - Returned %d",x);
          return;
       }

	return;

}




