#include "stdinc.h"

#include "core.h"

#define TIMEBUF         100 /* Accepts time formatting */

/* These items are for the timediff() function, used to process time
** quantities (3 days, 45 minutes, 21 seconds)
*/
#define T_DAY       "day"
#define T_DAYS      "days"
#define T_HOUR      "hour"
#define T_HOURS     "hours"
#define T_MINUTE    "minute"
#define T_MINUTES   "minutes"
#define T_SECOND    "second"
#define T_SECONDS   "seconds"

static char current_uid[10];

char * strtolower (char *str)
{
    int cnt;
    static char str_lower[1024];

    memset(str_lower,0,sizeof(str_lower));

    for (cnt = 0; *str && cnt < sizeof(str_lower); cnt++, str++);
          str_lower[cnt] = tolower(*str);


    return (char *) str_lower;
}


/**
 * Since not every one has strlcpy and strlcat we need to create them if
 * they do not exist on the system
 */

#ifndef HAVE_STRLCAT
size_t strlcat(char *dst, const char *src, size_t siz)
{
  char *d = dst;
  const char *s = src;
  size_t n = siz, dlen;

  while (n-- != 0 && *d != '\0')
    d++;
  dlen = d - dst;
  n = siz - dlen;

  if (n == 0)
    return (dlen + strlen(s));
  while (*s != '\0')
  {
    if (n != 1)
    {
      *d++ = *s;
      n--;
    }
    s++;
  }
  *d = '\0';
  return (dlen + (s - src));       /* count does not include NUL */
}
#endif

#ifndef HAVE_STRLCPY
size_t strlcpy(char *dst, const char *src, size_t siz)
{
  char *d = dst;
  const char *s = src;
  size_t n = siz;
  /* Copy as many bytes as will fit */
  if (n != 0 && --n != 0)
  {
    do
    {
      if ((*d++ = *s++) == 0)
        break;
    }
    while (--n != 0);
  }
  /* Not enough room in dst, add NUL and traverse rest of src */
  if (n == 0)
  {
    if (siz != 0)
      *d = '\0';              /* NUL-terminate dst */
    while (*s++)
      ;
  }

  return (s - src - 1);    /* count does not include NUL */
}
#endif

void init_uid (void)
{
	int	i;

	for (i = 0; i < 3; i++)
		current_uid[i]	= CfgSettings.sid[i];

	for (i = 3; i < 9; i++)
		current_uid[i] = 'A';

	current_uid[9] = '\0';
}

char * generate_uid (void)
{
	int	i;

	for (i = 8; i > 3; i--)
	{
		if (current_uid[i] == 'Z')
		{
			current_uid[i] = '0';
			return current_uid;
		}
		else if (current_uid[i] != '9')
		{
			current_uid[i]++;
			return current_uid;
		}
		else
			current_uid[i] = 'A';
	}

	if (current_uid[3] == 'Z')
	{
		current_uid[i] = 'A';
		s_assert (0);
	}
	else
		current_uid[i]++;

	return current_uid;
}

void ExpandParv(char *dest, int max, int initial, int parc, char *parv[])
{
	 int i;

	if (parv[initial])
		strlcpy(dest, parv[initial], max);

	for (i = initial + 1; i < parc; i++)
	{
		strlcat(dest, " ", max);
		strlcat(dest, parv[i], max);
	}
}

int generictoken(char delim, int size, char *message, char **parv)
{
  char *next;
  int count;

  if (!message)
  {
    /* Something is seriously wrong...bail */
    parv[0] = NULL;
    return 0;
  }

  /* Now we take the beginning of message and find all the spaces...
  ** set them to \0 and use 'next' to go through the string
  */
  next = message;
  parv[0] = next;
  count = 1;

  while (*next)
  {
    /* This is fine here, since we don't have a :delimited
    ** parameter like tokenize
    */
    if (count == size)
    {
      /* We've reached our limit */
      //slog(DEFAULT_LOG, L_DEBUG, "Reached the size limit!");
      return count;
    }
    if (*next == delim)
    {
      *next = '\0';
      next++;
      /* Eat any additional delimiters */
      while (*next == delim)
        next++;
      /* If it's the end of the string, it's simply
      ** an extra space at the end.  Here we break.
      */
      if (*next == '\0')
        break;
      parv[count] = next;
      count++;
    }
    else
    {
      next++;
    }
  }

  return count;
}


uint32 hash(char *item)
{

  uint32  hashstr, i;

  for (hashstr=0, i=0; i < strlen(item); i++)
  {
    hashstr += tolower(item[i]);
    hashstr += (hashstr << 10);
    hashstr ^= (hashstr >> 6);
  }

  hashstr += (hashstr << 3);
  hashstr ^= (hashstr >> 11);
  hashstr += (hashstr << 15);
  return hashstr;

}


//aim to protect against poor hash table logic.
//borrowed from java hash table source - Twitch
unsigned int hash_safe(char *item, ub4 len)
{
    unsigned int i = (unsigned int) hash(item);
    i += ~(i << 9);
    i ^=  ((i >> 14) | (i << 18)); /* >>> */
    i +=  (i << 4);
    i ^=  ((i >> 10) | (i << 22)); /* >>> */
    return (unsigned int) (i % len);

}


/* print a period of seconds into human readable format */
char *timediff(time_t u)
{
  static char buf[TIMEBUF];

  if (u > 86400)
  {
    time_t days = u / 86400;

    snprintf(buf, TIMEBUF, "%ld %s, %02ld:%02ld",
             (long)days, (days == 1) ? T_DAY : T_DAYS,
             (long)((u / 3600) % 24), (long)((u / 60) % 60));
  }
  else if (u > 3600)
  {
    time_t hours = u / 3600;
    time_t minutes = (u / 60) % 60;

    snprintf(buf, TIMEBUF, "%ld %s, %ld %s",
             (long)hours, (hours == 1) ? T_HOUR : T_HOURS, (long)minutes,
             (minutes == 1) ? T_MINUTE : T_MINUTES);
  }
  else
  {
    time_t minutes = u / 60;
    time_t seconds = u % 60;

    snprintf(buf, TIMEBUF,
             "%ld %s, %ld %s",
             (long)minutes, (minutes == 1) ? T_MINUTE : T_MINUTES,
             (long)seconds, (seconds == 1) ? T_SECOND : T_SECONDS);
  }

  return buf;
}


/**
 * Change an unsigned string to a signed string, overwriting the original
 * string.
 * @param input string
 * @return output string, same as input string.
 */

char *str_signed(unsigned char *str)
{
    char *nstr;

    nstr = (char *) str;
    while (*str) {
        *nstr = (char) *str;
        str++;
        nstr++;
    }

    return nstr;
}



/* Equivalent to inet_ntoa */

void ntoa(struct in_addr addr, char *ipaddr, int len)
{

    unsigned char *bytes = (unsigned char *) &addr.s_addr;

    snprintf(ipaddr, len, "%u.%u.%u.%u", bytes[0], bytes[1], bytes[2],
             bytes[3]);

    return;

}

/** 
 * return an allocated string similar to inet_ntoa however
 * this doesnt take a lengh or char* argument however
 * this will only malloc up to 17 which is the lengh of an 
 * IP addr + 1 padding. so we dont piss away memory :)
 */



char *str_ntoa(struct in_addr addr)
{

    char ipaddr[17];

    unsigned char *bytes = (unsigned char *) &addr.s_addr;

    snprintf(ipaddr, sizeof(ipaddr), "%u.%u.%u.%u", bytes[0], bytes[1], bytes[2],
             bytes[3]);

    return (char*) strdup(ipaddr);
}

